@@ -1,5 +1,5 @@
 import pathlib
-from ._upath import Upath, make_a_method
+from ._upath import Upath
 from ._local import LocalUpath
 
 
@@ -15,7 +15,7 @@ def _resolve_local_path(p):
 
 class BlobUpath(Upath):  # pylint: disable=abstract-method
     @property
-    def _blob_name(self) -> str:
+    def blob_name(self) -> str:
         return self._path.lstrip('/')
 
     def download_dir(self, target, **kwargs) -> int:
@@ -46,12 +46,13 @@ class BlobUpath(Upath):  # pylint: disable=abstract-method
         an "empty directory" in blob stores.
         '''
         try:
-            next(self.riterdir())
+            next(self.iterdir())
             return True
         except StopIteration:
             return False
 
     def iterdir(self):
+        # A naive, inefficient implementation.
         p0 = self._path  # this could be '/'.
         if not p0.endswith('/'):
             p0 += '/'
@@ -74,9 +75,3 @@ class BlobUpath(Upath):  # pylint: disable=abstract-method
     def upload_file(self, source, **kwargs) -> int:
         s = _resolve_local_path(source)
         return self.import_file(s, **kwargs)
-
-
-for m in ('download_dir', 'download_file',
-          'upload_dir', 'upload_file',
-          ):
-    setattr(BlobUpath, f'a_{m}', make_a_method(m))
